/* libvisio
 * Copyright (C) 2011 Fridrich Strba <fridrich.strba@bluewin.ch>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02111-1301 USA
 */

#ifndef __VSDXSTYLES_H__
#define __VSDXSTYLES_H__

#include <map>
#include <vector>
#include <libwpg/libwpg.h>
#include "VSDXTypes.h"

namespace libvisio {

struct VSDXLineStyle
{
  VSDXLineStyle();
  VSDXLineStyle(double w, Colour col, unsigned short p, unsigned c)
  : width(w), colour(col), pattern(p), cap(c) {}
  VSDXLineStyle(const VSDXLineStyle &lineStyle)
  : width(lineStyle.width), colour(lineStyle.colour), pattern(lineStyle.pattern), cap(lineStyle.cap) {}
  ~VSDXLineStyle() {}
  double width;
  Colour colour;
  unsigned short pattern;
  unsigned short cap;
};

struct VSDXFillStyle
{
  VSDXFillStyle();
  VSDXFillStyle(unsigned short fgcId, unsigned short bgcId, unsigned short p, unsigned fga, unsigned bga, Colour sfgc, unsigned short shp, double shX, double shY)
  : fgColourId(fgcId), bgColourId(bgcId), pattern(p), fgTransparency(fga), bgTransparency(bga), shadowFgColour(sfgc), shadowPattern(shp), shadowOffsetX(shX), shadowOffsetY(shY) {}
  VSDXFillStyle(const VSDXFillStyle &fillStyle)
  : fgColourId(fillStyle.fgColourId), bgColourId(fillStyle.bgColourId), pattern(fillStyle.pattern),
    fgTransparency(fillStyle.fgTransparency), bgTransparency(fillStyle.bgTransparency), shadowFgColour(fillStyle.shadowFgColour),
    shadowPattern(fillStyle.shadowPattern), shadowOffsetX(fillStyle.shadowOffsetX), shadowOffsetY(fillStyle.shadowOffsetY) {}
  ~VSDXFillStyle() {}
  unsigned short fgColourId;
  //  Colour fgColour;
  unsigned short bgColourId;
  //  Colour bgColour;
  unsigned short pattern;

  unsigned fgTransparency;
  unsigned bgTransparency;

  Colour shadowFgColour;
  unsigned short shadowPattern;
  double shadowOffsetX;
  double shadowOffsetY;
};

struct VSDXTextStyle
{
  VSDXTextStyle();
  VSDXTextStyle(CharFormat &f) : format(f) {} 
  CharFormat format;
};

class VSDXStyles
{
  public:
    VSDXStyles();
    ~VSDXStyles();
	void addLineStyle(unsigned lineStyleIndex, VSDXLineStyle *lineStyle);
	void addFillStyle(unsigned fillStyleIndex, VSDXFillStyle *fillStyle);
	void addTextStyle(unsigned textStyleIndex, VSDXTextStyle *textStyle);
	
	void addLineStyleMaster(unsigned lineStyleIndex, unsigned lineStyleMaster);
	void addFillStyleMaster(unsigned fillStyleIndex, unsigned fillStyleMaster);
	void addTextStyleMaster(unsigned textStyleIndex, unsigned textStyleMaster);
	
	const VSDXLineStyle getLineStyle(unsigned lineStyleIndex) const;
	const VSDXFillStyle getFillStyle(unsigned fillStyleIndex) const;
	const VSDXTextStyle getTextStyle(unsigned textStyleIndex) const;
  private:
    std::map<unsigned, VSDXLineStyle> m_lineStyles;
    std::map<unsigned, VSDXFillStyle> m_fillStyles;
    std::map<unsigned, VSDXTextStyle> m_textStyles;
    std::map<unsigned, unsigned> m_lineStyleMasters;
    std::map<unsigned, unsigned> m_fillStyleMasters;
    std::map<unsigned, unsigned> m_textStyleMasters;
};


} // namespace libvisio

#endif // __VSDXSTYLES_H__
